@@ -97,12 +97,10 @@
 #' 
 #' @useDynLib matchingMarkets, .registration = TRUE 
 #' 
-#' @import partitions stats
-#' @importFrom Rcpp evalCpp 
+#' @import partitions stats RcppProgress
+#' @importFrom Rcpp evalCpp
 #' @importFrom utils setTxtProgressBar txtProgressBar
 #' 
-#' @aliases stabitSel2
-#' 
 #' @details 
 #' Operators for variable transformations in \code{selection} and \code{outcome} arguments.
 #' \describe{
@@ -553,7 +551,7 @@ stabit <- function(x, m.id="m.id", g.id="g.id", R="R", selection=NULL, outcome=N
     # ----------------------------------------------------------------------------- 
     
     model.frame <- unlistData(x=data)
-    model.frame
+    list(OUT=model.frame$OUT, SEL=model.frame$SEL, combs=combs)
     #return(list(model.list=data, model.frame=model.frame))
     
   }
@@ -602,17 +600,17 @@ unlistData <- function(x){
   x$D <- x$D[which(h>1)] # for 2-group markets only
   if(length(unlist(x$V)) == length(g.id)){ # if simulation = TRUE in model.matrix
     if(is.null(dim(x$W[[1]]))){ # only one variable in W
-      SEL <- with(x, data.frame( m.id, g.id, do.call(c,W), D=do.call(c,D),
-                                 V=do.call(c,V), eta=do.call(c,eta) ))
+      SEL <- with(x, data.frame( m.id, g.id, reg.id=na.omit(do.call(c,P)), do.call(c,W), 
+                                 D=do.call(c,D), V=do.call(c,V), eta=do.call(c,eta) ))
     } else{
-      SEL <- with(x, data.frame( m.id, g.id, do.call(rbind,W), D=do.call(c,D),
-                                 V=do.call(c,V), eta=do.call(c,eta) ))      
+      SEL <- with(x, data.frame( m.id, g.id, reg.id=na.omit(do.call(c,P)), do.call(rbind,W), 
+                                 D=do.call(c,D), V=do.call(c,V), eta=do.call(c,eta) ))      
     }
   } else{
     if(is.null(dim(x$W[[1]]))){ # only one variable in W
-      SEL    <- with(x, data.frame( m.id, g.id, do.call(c,W), D=do.call(c,D) ))    
+      SEL    <- with(x, data.frame( m.id, g.id, reg.id=na.omit(do.call(c,P)), do.call(c,W), D=do.call(c,D) ))    
     } else{
-      SEL    <- with(x, data.frame( m.id, g.id, do.call(rbind,W), D=do.call(c,D) ))    
+      SEL    <- with(x, data.frame( m.id, g.id, reg.id=na.omit(do.call(c,P)), do.call(rbind,W), D=do.call(c,D) ))    
     }
   }
   
